setlocal ts=2 sw=2 expandtab

"set background=light
"set guifont=Neep\ Medium\ Semi-Condensed\ 18

syntax on
filetype plugin indent on


" tags
call CmdAlias('hasktags', '!hasktags -c .<CR>')

" from
" https://raw.githubusercontent.com/begriffs/haskell-vim-now/master/git-hscope
call CmdAlias('codex', 'call system("haskell-ctags")<CR><CR>call LoadHscope()<CR>')


map <leader>tg :codex<CR>
set tags=tags;/,codex.tags;/

nnoremap <leader>gd <C-]>
nnoremap <silent> <leader>cgd :cs find g <C-R>=expand("<cword>")<CR><CR>
nnoremap <silent> <C-\> :cs find c <C-R>=expand("<cword>")<CR><CR>

set cscopeprg=do_at_stack_root\ hscope
set csre
set csto=1 " search codex tags first
set nocst
" set cscopequickfix=s-,c-,d-,i-,t-,e-,a-

function! LoadHscope()
	set nocscopeverbose " suppress 'duplicate connection' error
	let hsf = findfile("hscope.out", ".;")
	if filereadable(hsf)
		exe "cs add " . hsf
	elseif $HSCOPE_DB != ""
		cs add $HSCOPE_DB
	endif
	set cscopeverbose
endfunction
au BufEnter /*.hs call LoadHscope()




"hoogle
nnoremap <leader>ho :Hoogle<CR>
nnoremap <leader>hc :HoogleClose<CR>


" haskell-vim
let g:haskell_classic_highlighting = 1
let g:haskell_indent_disable = 1
" let g:haskell_enable_quantification = 1
" let g:haskell_enable_recursivedo = 1
" let g:haskell_enable_arrowsyntax = 1
" let g:haskell_enable_pattern_synonyms = 1
" let g:haskell_enable_typeroles = 1
" let g:haskell_indent_if = 3
" let g:haskell_indent_case = 5
" let g:haskell_indent_let = 4
" let g:haskell_indent_where = 6
" let g:haskell_indent_do = 3
" let g:haskell_indent_in = 1


" haskellmode-vim
function! HaskellDocCurrentWord()
        let word = expand("<cword>")
        exe "IDoc " . word
endfunction

" done by LSP now
nmap <F7> :call HaskellDocCurrentWord()<CR><CR>

nmap <silent> <F3> :silent update <bar> HsimportModule<CR>
nmap <silent> <F4> :silent update <bar> HsimportSymbol<CR>


" liquid-types
let g:vim_annotations_offset = '/.liquid/'


" autocmd BufWritePost *.hs call s:check_and_lint()
" function! s:check_and_lint()
  " let l:path = expand('%:p')
  " let l:qflist = ghcmod#make('check', l:path)
  " call extend(l:qflist, ghcmod#make('lint', l:path))
  " call setqflist(l:qflist)
  " cwindow
  " if empty(l:qflist)
    " echo "No errors found"
  " endif
" endfunction



" LSP
let g:LanguageClient_autoStart = 1
let g:LanguageClient_serverCommands = {
	\ 'haskell': ['hie-wrapper', '--lsp', '-d', '-l', $HOME.'/lang-server.log'],
	\ }
" we use ALE instead
let g:LanguageClient_diagnosticsEnable = 0

nnoremap <leader>lc :call LanguageClient_contextMenu()<CR>
nnoremap <silent> gd :call LanguageClient#textDocument_definition()<CR>
nnoremap <silent> <C-F6> :call LanguageClient#textDocument_hover()<CR>
vnoremap <silent> <C-F6> :call LanguageClient#textDocument_hover()<CR>
nnoremap <silent> <F8> :call LanguageClient#textDocument_rename()<CR>

" deoplete
call deoplete#custom#option('sources',{
	\ '_': ['buffer'],
	\ 'haskell': ['neosnippet', 'buffer', 'file', 'LanguageClient']
	\ })
" inoremap <expr><tab> pumvisible() ? "\<c-n>" : "\<tab>"
call deoplete#enable()
call deoplete#enable_logging('DEBUG', $HOME . '/deoplete.log')

let g:ghcmod_hlint_options = ['--ignore=Eta reduce $']

" ALE
let g:ale_enabled = 1
let g:ale_linters = {'haskell':['ghc-mod', 'hdevtools'], 'c':['clang']}
" let g:ale_linters = {'haskell':['ghc-mod', 'hdevtools', 'argon'], 'c':['clang']}
" let g:ale_fixers = {
	" \    'haskell': ['brittany'],
	" \}
let g:ale_haskell_hdevtools_options = "-g '-Wall' -g '-Wno-orphans'"
let g:ale_haskell_argon_error_level = 14
let g:ale_haskell_argon_warn_level = 10
let g:ale_haskell_argon_info_level = 6

" completion
"
" neco-ghc
" let g:necoghc_enable_detailed_browse = 1
" let g:haskellmode_completion_ghc = 0
" let g:necoghc_enable_detailed_browse = 1
" autocmd FileType haskell setlocal omnifunc=necoghc#omnifunc
" let g:ycm_semantic_triggers = {'haskell' : ['.']}




" clear search
nmap <F2> :noh<CR>:GhcModTypeClear<CR>
imap <F2> <C-O>:noh<CR>:GhcModTypeClear<CR>

" unmap <F3>
" unmap <F4>

nmap <F6> :GhcModType<CR>

" for intero
" if has("nvim")
	" let g:ale_enabled = 0

	" " let g:intero_backend = {
			" " \ 'command': 'cabal new-repl',
			" " \ 'options': '',
			" " \ 'cwd': expand('%:p:h'),
			" " \}

	" " Intero starts automatically. Set this if you'd like to prevent that.
	" let g:intero_start_immediately = 1

	" " Enable type information on hover (when holding cursor at point for ~1 second).
	" let g:intero_type_on_hover = 1

	" " Change the intero window size; default is 10.
	" let g:intero_window_size = 15

	" " Sets the intero window to split vertically; default is horizontal
	" let g:intero_vertical_split = 1

	" " OPTIONAL: Make the update time shorter, so the type info will trigger faster.
	" set updatetime=1000

	" map <silent> <leader>t <Plug>InteroGenericType
" endif
